/*
 * Standard image size definitions
 *
 * Copyright (C) 2013, Sylwester Nawrocki <sylvester.nawrocki@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef _IMAGE_SIZES_H
#define _IMAGE_SIZES_H

#define CIF_WIDTH	352
#define CIF_HEIGHT	288

#define QCIF_WIDTH	176
#define QCIF_HEIGHT	144

#define QQCIF_WIDTH	88
#define QQCIF_HEIGHT	72

#define QQVGA_WIDTH	160
#define QQVGA_HEIGHT	120

#define QVGA_WIDTH	320
#define QVGA_HEIGHT	240

#define SXGA_WIDTH	1280
#define SXGA_HEIGHT	1024

#define VGA_WIDTH	640
#define VGA_HEIGHT	480

#endif /* _IMAGE_SIZES_H */
